﻿--[[
Author: Mantrid
https://fs.mantrid.net/
Copyright: All rights reserved
LEGAL NOTICE: This mod, and this file in particular, are only authorised for distribution by GIANTS Software. All other entities agree to pay the sum of AU$1,000,000 for distribution rights.
Aspiring developers: I suggest you look elsewhere if you want to know how to do things right. This is my 2nd LUA mod, and it's full of not-the-right-way methods, and pointless things. I intend to refactor it in the not-too-distant future
Experienced developers: It's my first day :)
]]

EngineStarter = {
	isES = true,
	-- Default settings and placeholder for restored settings
	isEnabled_throttleStart = true,
	isEnabled_quickStart = false,
	isEnabled_showWarnings = true,
	isEnabled_turnOnVehicle = true,
	isEnabled_other = true,

	multiShared_values = {g_i18n:getText("option_Multi_NotAllowed"), g_i18n:getText("option_Multi_AlwaysAllowed"), g_i18n:getText("option_Multi_AutoStartEngine")},
	multiShared_steerable = 2,
	multiShared_cylindered = 2,
	multiShared_foldable = 2,
	multiShared_foldMiddle = 2,
	multiShared_pipe = 2,
	multiShared_crabSteering = 2,
	multiShared_dischargeable = 3,
	isEnabled_stopDischargeOnDeactivate = false

	-- This feature was disabled, since the base-game timer stops counting after 2 seconds, and never resets the countdown.
	-- multi_motorStopTimerDuration = 4,
	-- motorStopTimerDuration_values = {'6', '60', '120', '300', '600', '900', '1800', '2700', '3600', '7200'}
}

local EngineStarter_mt = Class(EngineStarter)

local consoleLogging = true -- set to false if you don't like the log spam :)

function EngineStarter:new(modDirectory, modName, customMt)
	local self = setmetatable({}, customMt or EngineStarter_mt)
	self.foo = bar

    self.modDirectory = modDirectory
    self.modName = modName
	
	local modDescFile = loadXMLFile("modDesc", modDirectory .. "modDesc.xml")
	self.title = getXMLString(modDescFile, "modDesc.title.en")
	self.author = getXMLString(modDescFile, "modDesc.author")
	self.version = getXMLString(modDescFile, "modDesc.version")
	delete(modDescFile)
	
	self.addSettingsPage(modDirectory)
	
	if consoleLogging then Logging.info("Loaded mod " .. self.title .. " v" .. self.version .. " by " .. self.author) end
	return self
end

function EngineStarter.addSettingsPage(modDirectory)
	local engineStarterSettingsFrame = EngineStarterSettingsFrame.new()
	local engineStarterSettingsFrameReference = EngineStarterSettingsFrameReference.new()

	g_gui:loadGui(modDirectory .. "gui/EngineStarterSettingsFrame.xml", "EngineStarterSettingsFrame", engineStarterSettingsFrame, true)
	g_gui:loadGui(modDirectory .. "gui/EngineStarterSettingsFrameReference.xml", "EngineStarterSettingsFrameReference", engineStarterSettingsFrameReference)

	local inGameMenu = g_currentMission.inGameMenu
	local pageEngineStarterSettings = engineStarterSettingsFrameReference.pageEngineStarterSettings

	if pageEngineStarterSettings ~= nil then
		local pagingElement = inGameMenu.pagingElement
		local index = pagingElement:getPageIndexByElement(inGameMenu.pageSettingsGeneral) + 1

		PagingElement:superClass().addElement(pagingElement, pageEngineStarterSettings)
		pagingElement:addPage(string.upper(pageEngineStarterSettings.name), pageEngineStarterSettings, g_i18n:getText("title_engineStarter"), index)

		inGameMenu:registerPage(pageEngineStarterSettings, index, inGameMenu:makeIsGeneralSettingsEnabledPredicate())
		inGameMenu:addPageTab(pageEngineStarterSettings, modDirectory .. "gui/ui_icon.png", GuiUtils.getUVs({0, 0, 64, 64}, {64, 64}))
		inGameMenu.pageEngineStarterSettings = pageEngineStarterSettings
	end
end

function EngineStarter:onMissionStart(mission)
	if consoleLogging then Logging.info(string.format("EngineStarter: %s (ver %s) is loaded", g_engineStarter.title, g_engineStarter.version)) end
	self.loadSettings()
	if self.isEnabled_throttleStart then 
		g_currentMission:setAutomaticMotorStartEnabled(false)
	end
end

function EngineStarter:loadMap(filename)
	if g_currentMission.inGameMenu.pageEngineStarterSettings ~= nil then
		g_currentMission.inGameMenu.pageEngineStarterSettings:initialize(self)
	end
end

-- Sneaksy validation that our addPoweredActionEvent hasn't been overwritten
function EngineStarter:addActionEvent()
	return false
end

function EngineStarter:addPoweredActionEvent(superFunc, actionEventsTable, inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
	if actionEventsTable == 'validate' then
		return true
	end
	local function newCallback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)

		local function startYourEngine(target)
		
			-- Check to make sure we have a motor, or go hunting for one
			if target.spec_motorized == nil then
				local vehicles = target.rootVehicle:getChildVehicles()

				for _, vehicle in pairs(vehicles) do
					if vehicle ~= target and vehicle.spec_motorized ~= nil then
						target = vehicle
					end
				end
				if target.spec_motorized == nil then
					-- Just case
					g_currentMission:showBlinkingWarning(warning, 2000)
					if consoleLogging then Logging.error("EngineStarter: Unable to find a motor!") end
				end
			end
		
			-- Change the warning, so the player knows the engine is starting
			local warning = g_i18n:getText("warning_motorStarting")
			local warningTime = target.spec_motorized.motorStartDuration -- We want the warning to only show while the engine is starting
			-- Check if QuickStart option is enabled
			if g_engineStarter.isEnabled_quickStart then
				warning = g_i18n:getText("warning_motorQuickStarted")
				if g_engineStarter.isEnabled_showWarnings then
					g_currentMission:showBlinkingWarning(warning, 500) -- Just a quick flash warning
				end
				local motorStartDuration = target.spec_motorized.motorStartDuration -- Backup
				target.spec_motorized.motorStartDuration = 0 -- Set
				target:startMotor() -- Do
				target.spec_motorized.motorStartDuration = motorStartDuration -- Restore
			else
				if g_engineStarter.isEnabled_showWarnings then
					g_currentMission:showBlinkingWarning(warning, warningTime)
				end
				target:startMotor()
			end
		
		end

		local function comesFromThisSpec(spec, actionName)
			if vehicle ~= nil then
				if vehicle[spec] ~= nil then
					if vehicle[spec].actionEvents ~= nil then
						if vehicle[spec].actionEvents[actionName] ~= nil then
							return true
						end
					end
				end
			end
			return false
		end

		local isPowered, warning = vehicle:getIsPowered()

		if isPowered then
			-- Nice and easy
			callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
		elseif inputValue ~= 0 and warning ~= nil then

			-- Check if the motor can't start because the clutch is out, and it's in gear
			if vehicle.rootVehicle and vehicle.rootVehicle.spec_motorized and warning == vehicle.rootVehicle.spec_motorized.clutchNoEngagedWarning then
				if consoleLogging then Logging.info("EngineStarter: Switching to neutral") end
				vehicle.rootVehicle.spec_motorized.motor.gear = 0
				vehicle.rootVehicle.spec_motorized.motor.targetGear = 0
				warning = g_i18n:getText("warning_motorNotStarted") -- Update the warning to reflect the change we made, so the next part knows the score
			end
			
			-- Check if the warning is that the motor is not started, but can start
			if warning == g_i18n:getText("warning_motorNotStarted") then
				-- Put our own warning in
				warning = g_i18n:getText("warning_motorRequired")
				-- Steering
				if actionName == "AXIS_MOVE_SIDE_VEHICLE" then 
					if g_engineStarter.multiShared_steerable == 2 then
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_steerable == 3 then
						startYourEngine(vehicle)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Throttle (or brakes)
				elseif actionName == "AXIS_ACCELERATE_VEHICLE" or actionName == "AXIS_BRAKE_VEHICLE" then
					if g_engineStarter.isEnabled_throttleStart then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Powered attachment
				elseif comesFromThisSpec('spec_turnOnVehicle', actionName) then
					if (g_engineStarter.isEnabled_turnOnVehicle) then
						if consoleLogging then Logging.info("EngineStarter: Starting engine for turnOnVehicle event") end
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Cylindered
				elseif comesFromThisSpec('spec_cylindered', actionName) or comesFromThisSpec('spec_frontloaderAttacher', actionName) then
					-- Logging.info("EngineStarter: Processing cylindered event and spamming your log!")
					if g_engineStarter.multiShared_cylindered == 2 then
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_cylindered == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Folding
				elseif comesFromThisSpec('spec_foldable', actionName) then
					if g_engineStarter.multiShared_foldable == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing fold event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_foldable == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Raise/Lower
				elseif vehicle.spec_foldable and vehicle.spec_foldable.foldMiddleInputButton == actionName then
					if g_engineStarter.multiShared_foldMiddle == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing raise/lower implement event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_foldMiddle == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Pipe
				elseif comesFromThisSpec('spec_pipe', actionName) then
					if g_engineStarter.multiShared_pipe == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing pipe control without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_pipe == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Crab Steering
				elseif comesFromThisSpec('spec_crabSteering', actionName) then
					if g_engineStarter.multiShared_crabSteering == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing Crab Steering toggle without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_crabSteering == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Discharge
				elseif comesFromThisSpec('spec_dischargeable', actionName) then
					if g_engineStarter.multiShared_dischargeable == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing discharge event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_dischargeable == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Other/unknown
				else
					if isEnabled_other then
						if consoleLogging then Logging.info("EngineStarter: Starting engine for unknown action " .. tostring(actionName)) end
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
						if consoleLogging then Logging.warning("EngineStarter: Unknown actionName = " .. tostring(actionName) .. " not being allowed") end
					end
				end
			else
				-- Motor can't start, so pass along the regular warning and don't do anything else
				g_currentMission:showBlinkingWarning(warning, 2000)
			end
			
		else
			-- Let 0 input values through, so they actually stop :)
			callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
		end
	end

	return self:addActionEvent(actionEventsTable, inputAction, target, newCallback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
end

function EngineStarter:onLeaveVehicle_prepend(wasEntered)
	-- Set the motor stop timer back to max, when leaving
	if self.spec_motorized then
		self.spec_motorized.motorStopTimer = self.spec_motorized.motorStopTimerDuration
	end
end
--[[
function EngineStarter:onLeaveVehicle_prepend(wasEntered)
	-- Set the motor stop timer
	if self.spec_motorized.motorStopTimerDuration ~= g_engineStarter.motorStopTimerDuration_values[g_engineStarter.multi_motorStopTimerDuration] * 1000 then
		local oldDuration = self.spec_motorized.motorStopTimerDuration
		self.spec_motorized.motorStopTimerDuration = g_engineStarter.motorStopTimerDuration_values[g_engineStarter.multi_motorStopTimerDuration] * 1000
		self.spec_motorized.motorStopTimer = self.spec_motorized.motorStopTimerDuration
		-- Logging.info("EngineStarter: Changed motor stop time from " .. oldDuration / 1000 .. " secs to " .. self.spec_motorized.motorStopTimerDuration / 1000 .. " secs");
	end
end

-- Fix a bug in base-game where the timer never gets reset, unless you're near the vehicle (but not in it), so tabbing around will cause motors to stop immediately, once the original timer gets below 0
function EngineStarter:onUpdateTick_prepend(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if isActiveForInputIgnoreSelection and self.spec_motorized.motorStopTimer ~= self.spec_motorized.motorStopTimerDuration then
		-- Logging.warning("EngineStarter: Resetting engine stop timer from " .. self.spec_motorized.motorStopTimer .. " to " .. self.spec_motorized.motorStopTimerDuration)
		self.spec_motorized.motorStopTimer = self.spec_motorized.motorStopTimerDuration
	end
end
]]

function EngineStarter:saveSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "engineStarter"
	local xmlFile = createXMLFile(key, savegameFolderPath .. "/mod_engineStarter.xml", key)

	setXMLBool(xmlFile, key .. ".throttleStartEnabled#value", g_engineStarter.isEnabled_throttleStart)
	setXMLBool(xmlFile, key .. ".quickStartEnabled#value", g_engineStarter.isEnabled_quickStart)
	setXMLBool(xmlFile, key .. ".showWarningsEnabled#value", g_engineStarter.isEnabled_showWarnings)
	setXMLBool(xmlFile, key .. ".turnOnVehicleEnabled#value", g_engineStarter.isEnabled_turnOnVehicle)
	setXMLBool(xmlFile, key .. ".stopDischargeOnDeactivateEnabled#value", g_engineStarter.isEnabled_stopDischargeOnDeactivate)
	-- setXMLInt(xmlFile, key .. ".motorStopTimerDuration#value", g_engineStarter.multi_motorStopTimerDuration)
	setXMLInt(xmlFile, key .. ".steerableMulti#value", g_engineStarter.multiShared_steerable)
	setXMLInt(xmlFile, key .. ".cylinderedMulti#value", g_engineStarter.multiShared_cylindered)
	setXMLInt(xmlFile, key .. ".foldableMulti#value", g_engineStarter.multiShared_foldable)
	setXMLInt(xmlFile, key .. ".foldMiddleMulti#value", g_engineStarter.multiShared_foldMiddle)
	setXMLInt(xmlFile, key .. ".pipeMulti#value", g_engineStarter.multiShared_pipe)
	setXMLInt(xmlFile, key .. ".crabSteeringMulti#value", g_engineStarter.multiShared_crabSteering)
	setXMLInt(xmlFile, key .. ".dischargeableMulti#value", g_engineStarter.multiShared_dischargeable)
	setXMLBool(xmlFile, key .. ".otherStartEnabled#value", g_engineStarter.isEnabled_other)

	saveXMLFile(xmlFile)

	delete(xmlFile)
end

function EngineStarter:loadSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "engineStarter"

	if fileExists(savegameFolderPath .. "/mod_engineStarter.xml") or fileExists(savegameFolderPath .. "/engineStarter.xml") then
		if consoleLogging then Logging.info("EngineStarter: Loading saved settings") end
		
		local xmlfile = 0
		if fileExists(savegameFolderPath .. "/mod_engineStarter.xml") then
			xmlFile = loadXMLFile(key, savegameFolderPath .. "/mod_engineStarter.xml")
		elseif fileExists(savegameFolderPath .. "/engineStarter.xml") then
			xmlFile = loadXMLFile(key, savegameFolderPath .. "/engineStarter.xml")
		end

		g_engineStarter.isEnabled_throttleStart = Utils.getNoNil(getXMLBool(xmlFile, key .. ".throttleStartEnabled#value"), g_engineStarter.isEnabled_throttleStart)
		g_engineStarter.isEnabled_quickStart = Utils.getNoNil(getXMLBool(xmlFile, key .. ".quickStartEnabled#value"), g_engineStarter.isEnabled_quickStart)
		g_engineStarter.isEnabled_showWarnings = Utils.getNoNil(getXMLBool(xmlFile, key .. ".showWarningsEnabled#value"), g_engineStarter.isEnabled_showWarnings)
		g_engineStarter.isEnabled_turnOnVehicle = Utils.getNoNil(getXMLBool(xmlFile, key .. ".turnOnVehicleEnabled#value"), g_engineStarter.isEnabled_turnOnVehicle)
		g_engineStarter.isEnabled_stopDischargeOnDeactivate = Utils.getNoNil(getXMLBool(xmlFile, key .. ".stopDischargeOnDeactivateEnabled#value"), g_engineStarter.isEnabled_stopDischargeOnDeactivate)
		-- g_engineStarter.multi_motorStopTimerDuration = Utils.getNoNil(getXMLInt(xmlFile, key .. ".motorStopTimerDuration#value"), g_engineStarter.multi_motorStopTimerDuration)
		g_engineStarter.multiShared_steerable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".steerableMulti#value"), g_engineStarter.multiShared_steerable)
		g_engineStarter.multiShared_cylindered = Utils.getNoNil(getXMLInt(xmlFile, key .. ".cylinderedMulti#value"), g_engineStarter.multiShared_cylindered)
		g_engineStarter.multiShared_foldable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".foldableMulti#value"), g_engineStarter.multiShared_foldable)
		g_engineStarter.multiShared_foldMiddle = Utils.getNoNil(getXMLInt(xmlFile, key .. ".foldMiddleMulti#value"), g_engineStarter.multiShared_foldMiddle)
		g_engineStarter.multiShared_pipe = Utils.getNoNil(getXMLInt(xmlFile, key .. ".pipeMulti#value"), g_engineStarter.multiShared_pipe)
		g_engineStarter.multiShared_crabSteering = Utils.getNoNil(getXMLInt(xmlFile, key .. ".crabSteeringMulti#value"), g_engineStarter.multiShared_crabSteering)
		g_engineStarter.multiShared_dischargeable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".dischargeableMulti#value"), g_engineStarter.multiShared_dischargeable)
		g_engineStarter.isEnabled_other = Utils.getNoNil(getXMLBool(xmlFile, key .. ".otherStartEnabled#value"), g_engineStarter.isEnabled_other)

		delete(xmlFile)
	end
end

function EngineStarter:updateBaseGameAutoStart()
	-- Disable base-game AutoStart if we're set to start on throttle
	local checkAutoMotorStart = g_currentMission.inGameMenu.pageSettingsGame.checkAutoMotorStart
	if g_engineStarter.isEnabled_throttleStart then
		checkAutoMotorStart:setIsChecked(false)
		checkAutoMotorStart.elements[6]:setText(g_i18n:getText("notice_AutoMotorStartDisabled"))
	else
		checkAutoMotorStart.elements[6]:setText(g_i18n:getText("toolTip_automaticMotorStart"))
	end

	checkAutoMotorStart:setDisabled(g_engineStarter.isEnabled_throttleStart)
end

addModEventListener(EngineStarter)
