EngineStarterSettingsFrame = {}
EngineStarterSettingsFrame.CONTROLS = {
	"settingsContainer",
	"boxLayout",
	"throttleStart",
	"quickStart",
	"showWarnings",
	"turnOnVehicle",
	"steerable",
	"cylindered",
	"foldable",
	"foldMiddle",
	"pipe",
	"crabSteering",
	"dischargeable",
	"stopDischargeOnDeactivate",
	"startForOthers"
}

local EngineStarterSettingsFrame_mt = Class(EngineStarterSettingsFrame, TabbedMenuFrameElement)

function EngineStarterSettingsFrame.new(subclass_mt)
	local self = TabbedMenuFrameElement.new(nil, subclass_mt or EngineStarterSettingsFrame_mt)

	self:registerControls(EngineStarterSettingsFrame.CONTROLS)
	
	self.optionMapping = {}
	self.isDirty = false

	return self
end

function EngineStarterSettingsFrame:onFrameOpen(element)
	EngineStarterSettingsFrame:superClass().onFrameOpen(self)
	self:updateGameSettings()
	
	self.boxLayout:invalidateLayout()

	if FocusManager:getFocusedElement() == nil then
		-- self:setSoundSuppressed(true)
		FocusManager:setFocus(self.boxLayout)
		-- self:setSoundSuppressed(false)
	end
end

function EngineStarterSettingsFrame:onFrameClose()
	EngineStarterSettingsFrame:superClass().onFrameClose(self)

	if self.isDirty then
		EngineStarter:saveSettings()
		self.isDirty = false
	end
end

function EngineStarterSettingsFrame:initialize(pageMapOverview, onClickBackCallback)
	self.pageMapOverview = pageMapOverview

	self:assignStaticTexts()
	self:updateButtons()

	self.boxLayout.wrapAround = false
	local firstSettingElement = self.boxLayout.elements[1]
	local lastSettingElement = self.boxLayout.elements[#self.boxLayout.elements]
	self.onClickBackCallback = onClickBackCallback or NO_CALLBACK
end

local function NO_CALLBACK()
end

function EngineStarterSettingsFrame:updateGameSettings()
	self.throttleStart:setIsChecked(g_engineStarter.isEnabled_throttleStart)
	self.quickStart:setIsChecked(g_engineStarter.isEnabled_quickStart)
	self.showWarnings:setIsChecked(g_engineStarter.isEnabled_showWarnings)
	self.turnOnVehicle:setIsChecked(g_engineStarter.isEnabled_turnOnVehicle)
	-- self.motorStopTimerDuration:setState(g_engineStarter.multi_motorStopTimerDuration)
	self.steerable:setState(g_engineStarter.multiShared_steerable)
	self.cylindered:setState(g_engineStarter.multiShared_cylindered)
	self.foldable:setState(g_engineStarter.multiShared_foldable)
	self.foldMiddle:setState(g_engineStarter.multiShared_foldMiddle)
	self.pipe:setState(g_engineStarter.multiShared_pipe)
	self.crabSteering:setState(g_engineStarter.multiShared_crabSteering)
	self.dischargeable:setState(g_engineStarter.multiShared_dischargeable)
	self.stopDischargeOnDeactivate:setIsChecked(g_engineStarter.isEnabled_stopDischargeOnDeactivate)
	self.startForOthers:setIsChecked(g_engineStarter.isEnabled_other)

	self.quickStart:setDisabled(not g_engineStarter.isEnabled_throttleStart)
	EngineStarter:updateBaseGameAutoStart()
	
	self.isDirty = false
end

function EngineStarterSettingsFrame:setMissionInfo(missionInfo)
	self.missionInfo = missionInfo
end


function EngineStarterSettingsFrame:updateButtons()
	self.menuButtonInfo = {
		self.backButtonInfo,
		self.saveButton,
		self.quitButton
	}

	-- self:setMenuButtonInfoDirty()
end

function EngineStarterSettingsFrame:assignStaticTexts()
	local textsWarnOrAuto = { g_i18n:getText("option_Multi_NotAllowed"), g_i18n:getText("option_Multi_AutoStartEngine") }
	local textsWarnAlwaysAuto = { g_i18n:getText("option_Multi_NotAllowed"), g_i18n:getText("option_Multi_AlwaysAllowed"), g_i18n:getText("option_Multi_AutoStartEngine") }
	local textsNoYes = { g_i18n:getText("ui_no"), g_i18n:getText("ui_yes") }

	self.throttleStart:setTexts(textsNoYes)
	self.quickStart:setTexts(textsNoYes)
	self.showWarnings:setTexts(textsNoYes)

	self.turnOnVehicle:setTexts(textsWarnOrAuto)
	self.steerable:setTexts(textsWarnAlwaysAuto)
	self.cylindered:setTexts(textsWarnAlwaysAuto)
	self.foldable:setTexts(textsWarnAlwaysAuto)
	self.foldMiddle:setTexts(textsWarnAlwaysAuto)
	self.pipe:setTexts(textsWarnAlwaysAuto)
	self.crabSteering:setTexts(textsWarnAlwaysAuto)
	self.dischargeable:setTexts(textsWarnAlwaysAuto)
	self.stopDischargeOnDeactivate:setTexts(textsNoYes)
	self.startForOthers:setTexts(textsWarnOrAuto)

	-- local helperTexts = {
		-- g_i18n:getText(EngineStarterSettingsFrame.L10N_SYMBOL.OFF),
		-- g_i18n:getText(EngineStarterSettingsFrame.L10N_SYMBOL.BUY)
	-- }

	-- self.checkHelperRefillFuel:setTexts(helperTexts)
end


function EngineStarterSettingsFrame:onClick_throttleStart(state)
    g_engineStarter.isEnabled_throttleStart = state == CheckedOptionElement.STATE_CHECKED
	
	-- Set base-game AutoStart and QuickStart to off and disable
	if (g_engineStarter.isEnabled_throttleStart) then
		g_currentMission.missionInfo.automaticMotorStartEnabled = false -- Directly set the settings (since the menu item callback won't be called)
		self.quickStart:setIsChecked(g_engineStarter.isEnabled_quickStart)
	else
		self.quickStart:setIsChecked(false) -- uncheck the menu item (we don't care about the actual setting. It's just to highlight to players that they can't use it
	end

	self.quickStart:setDisabled(not g_engineStarter.isEnabled_throttleStart)
	
	EngineStarter:updateBaseGameAutoStart()
	self.isDirty = true
end


function EngineStarterSettingsFrame:onClick_quickStart(state)
    g_engineStarter.isEnabled_quickStart = state == CheckedOptionElement.STATE_CHECKED
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_showWarnings(state)
    g_engineStarter.isEnabled_showWarnings = state == CheckedOptionElement.STATE_CHECKED
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_turnOnVehicle(state)
    g_engineStarter.isEnabled_turnOnVehicle = state == CheckedOptionElement.STATE_CHECKED
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_steerable(state)
	g_engineStarter.multiShared_steerable = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_cylindered(state)
	g_engineStarter.multiShared_cylindered = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_foldable(state)
	g_engineStarter.multiShared_foldable = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_foldMiddle(state)
	g_engineStarter.multiShared_foldMiddle = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_pipe(state)
	g_engineStarter.multiShared_pipe = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_crabSteering(state)
	g_engineStarter.multiShared_crabSteering = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_dischargeable(state)
	g_engineStarter.multiShared_dischargeable = state
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_stopDischargeOnDeactivate(state)
	g_engineStarter.isEnabled_stopDischargeOnDeactivate = state == CheckedOptionElement.STATE_CHECKED
	self.isDirty = true
end

function EngineStarterSettingsFrame:onClick_startForOthers(state)
	g_engineStarter.isEnabled_other = state == CheckedOptionElement.STATE_CHECKED
	self.isDirty = true
end




EngineStarterSettingsFrame.PROFILE = {
	BUTTON_PAUSE = "ingameMenuSettingsPauseButton",
	BUTTON_UNPAUSE = "ingameMenuSettingsUnpauseButton"
}
EngineStarterSettingsFrame.L10N_SYMBOL = {
	UNPAUSE = "ui_unpause",
	BUY = "ui_buy",
	USAGE_HIGH = "setting_high",
	DIFFICULTY_EASY = "button_easy",
	PAUSE = "input_PAUSE",
	USAGE_LOW = "setting_fuelUsageLow",
	USAGE_DEFAULT = "setting_fuelUsageDefault",
	DIFFICULTY_NORMAL = "button_normal",
	DIFFICULTY_HARD = "button_hard",
	SUBSTITUTION_PREFIX = "$l10n_",
	OFF = "ui_off",
	GROWTH_RATE_TEMPLATE = "setting_plantGrowthRateState"
}
