--[[
Author: Mantrid
https://fs.mantrid.net/
Copyright: All rights reserved
LEGAL NOTICE: This mod is only authorised for distribution by GIANTS Software. All other entities agree to pay the sum of AU$1,000,000 for distribution rights.
Aspiring developers: The following code may or may not be a good example, but it has lots of comments! Look and learn, but don't steal :)
Experienced developers: It's my first day :)
]]

local function initMod(modDirectory, modName)
	source(Utils.getFilename('scripts/EngineStarter.lua', modDirectory))
	
    Mission00.load = Utils.prependedFunction(Mission00.load, function(mission)
		g_engineStarter = EngineStarter:new(modDirectory, modName)
		if g_globalMods ~= nil and g_globalMods.engineStarter == nil then
			g_globalMods.engineStarter = g_engineStarter
		end
	end)
	
    Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, function(mission)
		EngineStarter:onMissionStart(mission)
	end)
	
	Vehicle.addPoweredActionEvent = Utils.overwrittenFunction(Vehicle.addPoweredActionEvent, EngineStarter.addPoweredActionEvent)

	-- Lets us override stopping discharge on trailer disconnect
	Dischargeable.onDeactivate = Utils.overwrittenFunction(Dischargeable.onDeactivate, function (self, superFunc)
		if g_engineStarter.isEnabled_stopDischargeOnDeactivate == true then
			superFunc(self)
		end
	end)

	-- Lets us override stopping discharge on engine power off
	Dischargeable.onStateChange = Utils.overwrittenFunction(Dischargeable.onStateChange, function (self, superFunc, state, data)
		if g_engineStarter.isEnabled_stopDischargeOnDeactivate == true then
			if state == Vehicle.STATE_CHANGE_MOTOR_TURN_OFF and g_engineStarter.multiShared_dischargeable == 2 then
				-- Don't stop discharge if we're turning off the engine, but we can tip with the engine off anyway
			else
				superFunc(self, state, data)
			end
		end
	end)

    InGameMenuGameSettingsFrame.onFrameOpen = Utils.appendedFunction(InGameMenuGameSettingsFrame.onFrameOpen, EngineStarter.updateBaseGameAutoStart)

	FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, EngineStarter.saveSettings) -- Settings are saved live, but we need to do it here too, since the old save directory (with our xml) is now a backup
	Drivable.onLeaveVehicle = Utils.prependedFunction(Drivable.onLeaveVehicle, EngineStarter.onLeaveVehicle_prepend)
	-- Motorized.onUpdateTick = Utils.prependedFunction(Motorized.onUpdateTick, EngineStarter.onUpdateTick_prepend)

	TypeManager.finalizeTypes = Utils.prependedFunction(TypeManager.finalizeTypes, function (object)
		if object.typeName == "vehicle" then
			for typeName, typeEntry in pairs(object:getTypes()) do
				if SpecializationUtil.hasSpecialization(Drivable, typeEntry.specializations) or SpecializationUtil.hasSpecialization(Pipe, typeEntry.specializations) or SpecializationUtil.hasSpecialization(Dischargeable, typeEntry.specializations) then
					object:addSpecialization(typeName, modName .. ".engineStarterSpec")
				end
			end
		end
	end)

	source(Utils.getFilename("scripts/gui/EngineStarterSettingsFrame.lua", modDirectory))
	source(Utils.getFilename("scripts/gui/EngineStarterSettingsFrameReference.lua", modDirectory))
end

initMod(g_currentModDirectory, g_currentModName)